# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
import pylustrator

font = 10
letter_fontsize = font

mpl.rc('xtick', labelsize=font)
mpl.rc('ytick', labelsize=font)
mpl.rc('legend', fontsize=font)
mpl.rc('axes', labelsize=font, titlesize=font)
mpl.rc('mathtext', default="regular")
mpl.rc('figure', dpi=100)
mpl.rc('legend', frameon=False)
mpl.rc('font', family="Helvetica Neue")

letter_index = 0
def add_letter(ax = None, offset=0, offset2=0, letter=None):
    global letter_index

    if ax is None:
        ax = plt.gca()

    box = ax.get_position()
    x_off = (-0.5/2.54)/plt.gcf().get_figwidth()+offset
    if letter is None:
        letter = chr(ord("a") + letter_index)
        letter_index += 1
    plt.text(box.x0+x_off, box.y1+offset2, letter, fontproperties=get_letter_font_prop(), va="bottom", ha="center", transform=plt.gcf().transFigure, picker=True)

def get_letter_font_prop():
    from matplotlib.font_manager import FontProperties
    font = FontProperties()
    font.set_family("Helvetica Neue")
    font.set_weight("bold")
    font.set_size(letter_fontsize)
    return font

def add_letters(*args, **kwargs):
    for ax in plt.gcf().axes:
        add_letter(ax, *args, **kwargs)

cmap = pylustrator.LabColormap(["c", "b"], 10)

np.random.seed(7)


def logistic_function(x):
    p3 = (60 - 25) / np.log(19)
    y = (0.01 / (1 + np.exp(-1 * ((x - 60) / p3))))
    return y


def example_single_trail():
    # the sampling rate
    fs = 20000
    # 0.01s sampled with the rate
    t = np.arange(0, 0.01, 1 / fs)

    # mean and standard deviation
    mu, sigma = 0, 0.04

    wave = np.sqrt(2) * logistic_function(135) * np.sin(1000 * 2 * np.pi * t) * 1000

    single_trial = np.zeros(3650)
    single_trial[int(0.05 * fs):int(0.06 * fs)] = wave

    noise = np.random.normal(mu, sigma, 3650) * 1000

    time = np.arange(len(single_trial)) / 20 - 50

    return time, single_trial, noise


sound_level = np.array([-25, -15, -5, 5, 15, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85, 95, 105, 115, 125])

# Plot a, the response amplitude over the stimulus intensity
ax = plt.axes([0.1, 0.1, 0.1, 0.1], label="1")
intensities = np.arange(-40, 140)
ax.plot(intensities, 1000 * logistic_function(intensities), color=cmap(5))
ax.plot(sound_level, 1000 * logistic_function(sound_level), 's', color=cmap(5), ms=4)
ax.set_xlabel('stimulus intensity (dB)')
ax.set_ylabel('response amplitude, RMS')
ax.set_ylim(0, 10.5)


time, no_noise, noise = example_single_trail()
# Plot b, displaying the simulated signal
ax2 = plt.axes([0.2, 0.1, 0.1, 0.1], label="2")
ax2.plot(time, no_noise, color=cmap(2))
ax2.set_xlim(-10, 20)
ax2.set_xticklabels([])

# Plot c, displaying the simulated noise
ax3 = plt.axes([0.3, 0.1, 0.1, 0.1], label="3")
ax3.plot(time, noise, color=cmap(5))
ax3.set_xlim(-10, 20)
ax3.set_xticklabels([])
ax3.set_ylabel('raw signal')
ax3.set_ylim(-150, 150)
ax3.set_yticks([-100, 0, 100])

# Plot d, displaying the signal overlaid with the noise
ax4 = plt.axes([0.4, 0.1, 0.1, 0.1], label="4")
ax4.plot(time, no_noise + noise, color=cmap(8))
ax4.set_xlabel('time (ms)')
ax4.set_xlim(-10, 20)
ax4.set_ylim(-150, 150)
ax4.set_yticks([-100, 0, 100])


# load the artificial data
complete = np.loadtxt('response_amplitudes.txt')

# the first column is the intensity and the rest of the column is the data
stimulus_intensities = complete[:, 0]
wave_data = complete[:, 1:]

# Plot e, Signals for different stimulus intensities
ax5 = plt.axes([0.5, 0.1, 0.1, 0.1], label="5")
factor = 0.05
c_t_pre = 0.002
cmap1 = pylustrator.LabColormap(["c", "b"], len(complete[:, 0]))
times = ((np.arange(len(complete[0, 1::]))) / 20) - c_t_pre * 1000
yticks = []
for i in range(len(stimulus_intensities)):
    ax5.plot(times, wave_data[i] + factor * i, linestyle='-', color=cmap1(i), clip_on=False)
    yticks.append(factor * i)

# the red line to indicate the beginning of the stimulus
ax5.plot([0, 0], [0 - factor / 2, i * factor + factor], 'r--')

# the scale bar for the recorded voltage
y_off = 2 * factor
x = 10.2
length = 100
plt.plot([x, x], [y_off, y_off + length * 1e-3], clip_on=False, color="k")

# x limits and ticks
plt.xlim(- c_t_pre * 1e3 - 0.1, 10)
ax5.set_yticks(yticks[1::2])  # only every second label a major tick
ax5.set_yticks(yticks, minor=True)  # the rest are minor ticks
ax5.set_yticklabels(map(int, np.unique(stimulus_intensities)[1::2]))
ax5.set_xlabel('time (ms)')
ax5.set_ylabel('stimulus intensity (dB)')
ax5.set_ylim(0 - factor / 2, i * factor + factor)


# load the data for plot f
rms = np.loadtxt('averaged_response_amplitudes.txt')
params = np.loadtxt('averaged_response_amplitudes_baseline.txt')

# Plot f, response amplitude over the stimulus intensity
ax6 = plt.axes([0.6, 0.1, 0.1, 0.1], label="6")

# plot the data
ax6.plot(rms[:, 0], rms[:, 1] * 1000, 's', ms=4, color=cmap(5))
# plot the base level line
ax6.plot([0, 160], [params * 1000, params * 1000], 'k--', lw=0.8)

# label the plot
ax6.set_xlabel('stimulus intensity (dB)')
ax6.set_ylabel('response amplitude, RMS')
ax6.set_xlim(0, 160)

add_letters()

#% start: automatic generated code from pylustrator
fig = plt.figure(1)
fig.ax_dict = {ax.get_label(): ax for ax in fig.axes}
fig.ax_dict["1"].set_position([0.109677, 0.627800, 0.357552, 0.327984])
fig.ax_dict["5"].set_position([0.109677, 0.123305, 0.357552, 0.379661])
fig.set_size_inches(16.000000 / 2.54, 12.000000 / 2.54, forward=True)
fig.ax_dict["3"].text(0.5, 0.5, 'New Text', transform=fig.ax_dict["3"].transAxes)
fig.ax_dict["4"].text(0.5, 0.5, 'New Text', transform=fig.ax_dict["4"].transAxes)
fig.ax_dict["3"].text(0.5, 0.5, 'New Text', transform=fig.ax_dict["3"].transAxes)
fig.ax_dict["3"].texts[0].set_text("+")
fig.ax_dict["3"].texts[0].set_position([0.486295, 1.066404])  # Text: "+"
fig.ax_dict["3"].texts[0].set_ha("center")
fig.ax_dict["4"].text(0.5, 0.5, 'New Text', transform=fig.ax_dict["4"].transAxes)
fig.ax_dict["4"].texts[0].set_text("=")
fig.ax_dict["4"].texts[0].set_position([0.485428, 1.066404])  # Text: "="
fig.ax_dict["4"].texts[0].set_ha("center")
fig.ax_dict["4"].texts[0].set_weight("bold")
fig.ax_dict["4"].texts[0].set_fontsize(12)
fig.ax_dict["3"].texts[0].set_weight("bold")
fig.ax_dict["3"].texts[0].set_fontsize(12)
fig.ax_dict["2"].text(0.5, 0.5, 'New Text', transform=fig.ax_dict["2"].transAxes)
fig.ax_dict["2"].texts[0].set_text("pure response")
fig.ax_dict["2"].texts[0].set_color("#808080")
fig.ax_dict["2"].texts[0].set_ha("right")
fig.ax_dict["3"].text(0.5, 0.5, 'New Text', transform=fig.ax_dict["3"].transAxes)
fig.ax_dict["3"].texts[1].set_text("pure background")
fig.ax_dict["3"].texts[1].set_ha("right")
fig.ax_dict["3"].texts[1].set_color("#808080")
fig.ax_dict["4"].text(0.5, 0.5, 'New Text', transform=fig.ax_dict["4"].transAxes)
fig.ax_dict["4"].texts[1].set_text("superposition")
fig.ax_dict["4"].texts[1].set_ha("right")
fig.ax_dict["4"].texts[1].set_color("#808080")
fig.ax_dict["5"].text(0.5, 0.5, 'New Text', transform=fig.ax_dict["5"].transAxes, rotation=-90)
fig.ax_dict["5"].texts[0].set_ha("center")
fig.ax_dict["5"].texts[0].set_text("100 mV")
fig.ax_dict["5"].texts[0].set_position([1.055802, 0.226562])  # Text: "100 mV"
fig.ax_dict["4"].texts[2].set_position([1.484991, 1.339840])  # Text: "New Text"
fig.ax_dict["3"].texts[2].set_position([1.619232, 0.617187])  # Text: "New Text"
fig.ax_dict["6"].set_position([0.628300, 0.123305, 0.357552, 0.263136])
fig.ax_dict["4"].set_position([0.602863, 0.518461, 0.357552, 0.108475])
fig.ax_dict["3"].set_position([0.602863, 0.682885, 0.358174, 0.108475])
fig.ax_dict["2"].set_position([0.602863, 0.847309, 0.358174, 0.112712])
fig.ax_dict["3"].texts[1].set_position([1.060441, 1.066404])  # Text: "pure background"
fig.ax_dict["4"].texts[1].set_position([1.062286, 1.066404])  # Text: "superposition"
fig.ax_dict["2"].texts[0].set_position([1.060441, 0.781954])  # Text: "pure response"
fig.ax_dict["6"].texts[0].set_position([0.054580, 0.960021])  # Text: "a"
fig.ax_dict["6"].texts[4].set_position([0.054580, 0.513559])  # Text: "e"
fig.ax_dict["6"].texts[1].set_position([0.546176, 0.960021])  # Text: "b"
fig.ax_dict["6"].texts[2].set_position([0.546176, 0.798563])  # Text: "c"
fig.ax_dict["6"].texts[3].set_position([0.546176, 0.634139])  # Text: "d"
fig.ax_dict["6"].texts[5].set_position([0.574792, 0.413983])  # Text: "f"
fig.ax_dict["6"].set_xlim(0.0, 140.0)
fig.ax_dict["1"].spines['right'].set_visible(False)
fig.ax_dict["1"].spines['top'].set_visible(False)
fig.ax_dict["2"].spines['right'].set_visible(False)
fig.ax_dict["2"].spines['top'].set_visible(False)
fig.ax_dict["3"].spines['right'].set_visible(False)
fig.ax_dict["3"].spines['top'].set_visible(False)
fig.ax_dict["4"].spines['right'].set_visible(False)
fig.ax_dict["4"].spines['top'].set_visible(False)
fig.ax_dict["5"].spines['right'].set_visible(False)
fig.ax_dict["5"].spines['top'].set_visible(False)
fig.ax_dict["6"].spines['right'].set_visible(False)
fig.ax_dict["6"].spines['top'].set_visible(False)
#% end: automatic generated code from pylustrator

plt.savefig(__file__[:-3] + ".pdf", dpi=300)
plt.savefig(__file__[:-3] + ".png", dpi=300)

plt.show()
