# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
import pylustrator
from scipy.optimize import leastsq

font = 10
letter_fontsize = font

mpl.rc('xtick', labelsize=font)
mpl.rc('ytick', labelsize=font)
mpl.rc('legend', fontsize=font)
mpl.rc('axes', labelsize=font, titlesize=font)
mpl.rc('mathtext', default="regular")
mpl.rc('figure', dpi=100)
mpl.rc('legend', frameon=False)
mpl.rc('font', family="Helvetica Neue")

letter_index = 0
def add_letter(ax = None, offset=0, offset2=0, letter=None):
    global letter_index

    if ax is None:
        ax = plt.gca()

    box = ax.get_position()
    x_off = (-0.5/2.54)/plt.gcf().get_figwidth()+offset
    if letter is None:
        letter = chr(ord("a") + letter_index)
        letter_index += 1
    plt.text(box.x0+x_off, box.y1+offset2, letter, fontproperties=get_letter_font_prop(), va="bottom", ha="center", transform=plt.gcf().transFigure, picker=True)

def get_letter_font_prop():
    from matplotlib.font_manager import FontProperties
    font = FontProperties()
    font.set_family("Helvetica Condensed")
    font.set_weight("bold")
    font.set_size(letter_fontsize)
    return font

def add_letters(*args, **kwargs):
    for ax in plt.gcf().axes:
        add_letter(ax, *args, **kwargs)

def good_box(bp, color):
    for box in bp['boxes']:
        box.set(color=color, linewidth=1)
    for w in bp['whiskers']:
        w.set(color=color, linewidth=1)
    for f in bp['fliers']:
        f.set(mec="gray", alpha=0.5, linewidth=1, ms=1)
    for m in bp['medians']:
        m.set(linewidth=2, color="C3")
    for c in bp['caps']:
        c.set(linewidth=1, color='k')


def logistic_function_with_noiselevel(x, p1, p2, p3, const):
    y = np.sqrt((p1 / (1 + np.exp(-1 * ((x - p2) / p3)))) ** 2 + const ** 2)
    return y


def hard_sigmoid(x, p1, p2, p3):
    return p1 * (x - p2) * (x >= p2) * (p1 * (x - p2) < p3) + p3 * (p1 * (x - p2) >= p3)


def hard_sigmoid_with_noiselevel(x, p1, p2, p3, const):
    y = np.sqrt(const ** 2 + hard_sigmoid(x, p1, p2, p3) ** 2)
    # print y
    return y


def err_func(a, *args):
    rms, const = args
    for i in np.arange(len(rms[:, 1])):
        y = rms[:, 1] - hard_sigmoid_with_noiselevel(rms[:, 0], a[0], a[1], a[2], const)
    return y

def err_func_sigmoid(a, *args):
    rms, const = args
    for i in np.arange(len(rms[:, 1])):
        y = rms[:, 1] - logistic_function_with_noiselevel(rms[:, 0], a[0], a[1], a[2], const)
    return y


def fitting(rms, const, sigmoid):
    if sigmoid:
        startval = np.array([max(rms[:, 1]), np.mean(rms[:, 0]), 10])
        popt = leastsq(err_func_sigmoid, startval, args=(rms, const), maxfev=100000000)
        print("fit", popt)
    else:
        startval = np.array(
            [(max(rms[:, 1]) - min(rms[:, 1])) / (max(rms[:, 0]) - min(rms[:, 0])), np.mean(rms[:, 0]), max(rms[:, 1])])
        popt = leastsq(err_func, startval, args=(rms, const), maxfev=100000000)

    sq_err = 0
    popt = popt[0]
    popt3 = popt

    return popt, sq_err, popt3


def find_threshold(popt):
    threshold = -popt[2] * np.log(19) + popt[1]
    dynamic_range = 2 * popt[2] * np.log(19)
    return threshold, dynamic_range


def plotFile(filename, high_low="Low", sigmoid=False, cmap=None):
    # load the data
    rms = np.loadtxt(filename)
    # load the background noise level value
    background_estimation = np.loadtxt(filename.replace(".txt", "_background_estimation.txt"))

    ax = plt.gca()
    alphax = 0.7

    delete_point_count = 15

    x_for_fit = np.arange(-40, 180, 1)
    for delete_number in np.arange(delete_point_count):
        # remove data points from the lower or upper end
        if delete_number:
            if high_low == 'Low':
                rms_del = rms[delete_number:]
            else:
                rms_del = rms[:-delete_number - 1]
        else:
            rms_del = rms

        # fit the function to the data points
        fit, sq_err, popt3 = fitting(rms_del, background_estimation, sigmoid)

        # evaluate the function at the x values
        if sigmoid:
            y = logistic_function_with_noiselevel(x_for_fit, fit[0], fit[1], fit[2], background_estimation) * 1000
        else:
            y = hard_sigmoid_with_noiselevel(x_for_fit, fit[0], fit[1], fit[2], background_estimation) * 1000
        ax.plot(x_for_fit, y, color=cmap(delete_number), linewidth=1, alpha=alphax)

        # plot the data points
        ax.plot(rms_del[:, 0], rms_del[:, 1] * 1000, 's', color=cmap(delete_number), ms=3, zorder=5)

        # add a line for the threshold
        if sigmoid:
            ax.axvline(find_threshold(fit)[0], linestyle='--', color=cmap(delete_number))
        else:
            ax.axvline(fit[1], linestyle='--', color=cmap(delete_number))

    # add the labels
    ax.set_ylabel('RMS (mV)')
    ax.set_xlabel('sound level (dB)')
    ax.set_xlim([-40, 160])
    ax.set_ylim([0, 11])

def plotLeaveOut(filename, offset=0, cmap=None):
    thresh_vec_vec = np.loadtxt(filename)
    bp = plt.boxplot(thresh_vec_vec, whis=[5, 95], widths=0.3, positions=np.arange(thresh_vec_vec.shape[1])+offset)
    good_box(bp, cmap(7))
    plt.axhline(np.median(thresh_vec_vec[:, 0]), color=cmap(7), linestyle="--", lw=1)
    return bp

cmap_blue = pylustrator.LabColormap(["c", "b"], 15)
cmap_green = pylustrator.LabColormap(["y", "g"], 15)

fig = plt.figure(0)

plt.axes([0.1, 0.1, 0.8, 0.8], label="LowFitSigmoid")
plotFile("examples_logistic_low.txt", "Low", sigmoid=True, cmap=cmap_green)
plt.xticks([])
plt.xlabel("")
plt.gca().spines["bottom"].set_visible(False)

plt.axes([0.1, 0.1, 0.8, 0.8], label="HighFitSigmoid")
plotFile("examples_logistic_high.txt", "High", sigmoid=True, cmap=cmap_green)
plt.xticks([])
plt.xlabel("")
plt.gca().spines["bottom"].set_visible(False)

plt.axes([0.1, 0.1, 0.1, 0.1], label="LowFitRect")
plotFile("examples_hard_sigmoid_low.txt", "Low", cmap=cmap_blue)
plt.xticks([])
plt.xlabel("")
plt.gca().spines["bottom"].set_visible(False)
plt.axes([0.1, 0.1, 0.1, 0.1], label="HighFitRect")
plotFile("examples_hard_sigmoid_high.txt", "High", cmap=cmap_blue)

plt.axes([0.1, 0.1, 0.1, 0.1], label="1")
plotLeaveOut("thresholds_logistic_high.txt", -0.2, cmap=cmap_green)
plotLeaveOut("thresholds_hard_sigmoid_high.txt", 0.2, cmap=cmap_blue)

plt.ylim(0, 55)
plt.xlim(15, -1)
plt.xticks(np.arange(0, 15, 2)[::-1], ((np.arange(0, 15, 2)+3)*10).astype("int"))
plt.xlabel("highest sound level (dB)")
plt.ylabel("threshold (dB)")

plt.axes([0.1, 0.1, 0.1, 0.1], label="2")
plotLeaveOut("thresholds_logistic_low.txt", 0.2, cmap=cmap_green)
plotLeaveOut("thresholds_hard_sigmoid_low.txt", -0.2, cmap=cmap_blue)
plt.plot([0], [0], "-", color=cmap_green(7), label="5% criterion")
plt.plot([0], [0], "-", color=cmap_blue(7), label="knee criterion")
plt.legend()

plt.ylim(-20, 35)
plt.xlim(-1, 15)
plt.xticks(np.arange(0, 15, 2), ((np.arange(0, 15, 2)-2)*10).astype("int"))
plt.xlabel("lowest sound level (dB)")
plt.ylabel("threshold (dB)", labelpad=-3)

add_letters()

#% start: automatic generated code from pylustrator
fig = plt.figure(0)
import matplotlib as mpl
fig.ax_dict = {ax.get_label(): ax for ax in fig.axes}
fig.set_size_inches(15.980000/2.54, 12.000000/2.54, forward=True)
fig.ax_dict["1"].set_position([0.579427, 0.163438, 0.389348, 0.344139])
fig.ax_dict["1"].annotate('New Annotation', (15.0, 0.0), (7.0, 27.5), arrowprops=dict(arrowstyle='->'))
fig.ax_dict["1"].texts[0].set_position([5.366680, 4.813808])  # Text: "removal of points"
fig.ax_dict["1"].texts[0].set_text("removal of points")
fig.ax_dict["1"].texts[0].xy = (11.472028, 6.094798)
fig.ax_dict["1"].spines['right'].set_visible(False)
fig.ax_dict["1"].spines['top'].set_visible(False)
fig.ax_dict["2"].set_position([0.579427, 0.612555, 0.389348, 0.344139])
fig.ax_dict["2"].annotate('New Annotation', (-1.0, -20.0), (7.0, 7.5), arrowprops=dict(arrowstyle='->'), annotation_clip=False)
fig.ax_dict["2"].texts[0].set_position([0.015790, 0.967797])  # Text: "a"
fig.ax_dict["2"].texts[1].set_position([0.015790, 0.734746])  # Text: "b"
fig.ax_dict["2"].texts[2].set_position([0.537252, 0.941525])  # Text: "c"
fig.ax_dict["2"].texts[3].set_position([0.015790, 0.526271])  # Text: "d"
fig.ax_dict["2"].texts[4].set_position([0.015790, 0.295339])  # Text: "e"
fig.ax_dict["2"].texts[5].set_position([0.537252, 0.479661])  # Text: "f"
fig.ax_dict["2"].texts[6].set_position([1.446710, 35.603790])  # Text: "removal of points"
fig.ax_dict["2"].texts[6].set_text("removal of points")
fig.ax_dict["2"].texts[6].xy = (13.307887, 36.884780)
fig.ax_dict["2"].spines['right'].set_visible(False)
fig.ax_dict["2"].spines['top'].set_visible(False)
fig.ax_dict["HighFitRect"].set_position([0.079332, 0.117647, 0.389348, 0.196288])
fig.ax_dict["HighFitRect"].set_xlabel("sound level (dB)")
fig.ax_dict["HighFitRect"].spines['right'].set_visible(False)
fig.ax_dict["HighFitRect"].spines['top'].set_visible(False)
fig.ax_dict["HighFitSigmoid"].set_position([0.079332, 0.339739, 0.389348, 0.196288])
fig.ax_dict["HighFitSigmoid"].spines['right'].set_visible(False)
fig.ax_dict["HighFitSigmoid"].spines['top'].set_visible(False)
fig.ax_dict["LowFitRect"].set_position([0.079332, 0.561830, 0.389348, 0.196288])
fig.ax_dict["LowFitRect"].spines['right'].set_visible(False)
fig.ax_dict["LowFitRect"].spines['top'].set_visible(False)
fig.ax_dict["LowFitRect"].text(0.5, 0.5, 'New Text', transform=fig.ax_dict["LowFitRect"].transAxes)  # id=fig.ax_dict["LowFitRect"].texts[0].new
fig.ax_dict["LowFitRect"].texts[0].set_ha("right")
fig.ax_dict["LowFitRect"].texts[0].set_position([0.981829, 0.143812])  # Text: "extended
fig.ax_dict["LowFitRect"].texts[0].set_text("extended\nhard sigmoid fit")
fig.ax_dict["LowFitSigmoid"].set_position([0.079332, 0.783922, 0.389348, 0.196288])
fig.ax_dict["LowFitSigmoid"].set_ylabel("RMS (mV)")
fig.ax_dict["LowFitSigmoid"].spines['right'].set_visible(False)
fig.ax_dict["LowFitSigmoid"].spines['top'].set_visible(False)
fig.ax_dict["LowFitSigmoid"].text(0.5, 0.5, 'New Text', transform=fig.ax_dict["LowFitSigmoid"].transAxes)  # id=fig.ax_dict["LowFitSigmoid"].texts[0].new
fig.ax_dict["LowFitSigmoid"].texts[0].set_ha("right")
fig.ax_dict["LowFitSigmoid"].texts[0].set_position([0.981829, 0.133018])  # Text: "extended\nlogistic function fit"
fig.ax_dict["LowFitSigmoid"].texts[0].set_text("extended\nlogistic function fit")
#% end: automatic generated code from pylustrator

plt.savefig(__file__[:-3]+".pdf", dpi=300)
plt.savefig(__file__[:-3]+".png", dpi=300)

plt.show()
