# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
import pylustrator

font = 10
letter_fontsize = font

mpl.rc('xtick', labelsize=font)
mpl.rc('ytick', labelsize=font)
mpl.rc('legend', fontsize=font)
mpl.rc('axes', labelsize=font, titlesize=font)
mpl.rc('mathtext', default="regular")
mpl.rc('figure', dpi=100)
mpl.rc('legend', frameon=False)
mpl.rc('font', family="Helvetica Neue")

letter_index = 0
def add_letter(ax = None, offset=0, offset2=0, letter=None):
    global letter_index

    if ax is None:
        ax = plt.gca()

    box = ax.get_position()
    x_off = (-0.5/2.54)/plt.gcf().get_figwidth()+offset
    if letter is None:
        letter = chr(ord("a") + letter_index)
        letter_index += 1
    plt.text(box.x0+x_off, box.y1+offset2, letter, fontproperties=get_letter_font_prop(), va="bottom", ha="center", transform=plt.gcf().transFigure, picker=True)

def get_letter_font_prop():
    from matplotlib.font_manager import FontProperties
    font = FontProperties()
    font.set_family("Helvetica Neue")
    font.set_weight("bold")
    font.set_size(letter_fontsize)
    return font

def add_letters(*args, **kwargs):
    for ax in plt.gcf().axes:
        add_letter(ax, *args, **kwargs)

def hard_sigmoid(x, p1, p2, p3):
    return p1 * (x - p2) * (x >= p2) * (p1 * (x - p2) < p3) + p3 * (p1 * (x - p2) >= p3)


def hard_sigmoid_with_noiselevel(x, p1, p2, p3, const):
    y = np.sqrt(const ** 2 + hard_sigmoid(x, p1, p2, p3) ** 2)
    return y


fig_w = plt.figure(figsize=(16, 12))

for j in range(2):
    # load the data (plot (a) has the 2kHz data, plot (b) has the 4kHz data)
    if j == 0:
        complete = np.loadtxt('ABR_responses_2kHz.txt')
    else:
        complete = np.loadtxt('ABR_responses_4kHz.txt')

    # add the subplot
    ax_w = fig_w.add_subplot(3, 2, j + 1)
    factor = 0.01
    c_t_pre = 0.002
    c_t_post = 0.01

    # get a colormap
    cmap1 = pylustrator.LabColormap(["c", "b"], len(complete[:, 0]))
    color = cmap1(len(complete[:, 0]) // 2)

    # plot the response data
    x = ((np.arange(len(complete[0, 1::]))) / 20) - c_t_pre * 1000
    for i in range(len(complete[:, 0])):
        y = complete[i, 1::] + factor * i
        ax_w.plot(x, y, linestyle='-', color=cmap1(i), clip_on=False)
    # draw a line to illustrate the beginning of the stimulus
    ax_w.plot([0, 0], [0 - factor / 2, i * factor + factor], 'r--')

    # draw a scalebar for the response amplitude
    if j == 1:
        y_off = 2*factor
        x = 10.2
        length = 50
        plt.plot([x, x], [y_off, y_off+length*1e-3], clip_on=False, color="k")

    # set the ticks
    ax_w.set_yticks(np.arange(0, i * factor, factor*2))
    ax_w.set_ylim(0 - factor / 2, i * factor + factor)
    loudness = np.arange(40, 115, 10)
    ax_w.set_xlabel('time (ms)')
    # set the y label
    if j == 0:
        ax_w.set_ylabel('sound pressure\nlevel (dB SPL)')
        ax_w.set_yticklabels(list(map(str, loudness)))
    else:
        ax_w.set_yticklabels([])
    plt.xlim(- c_t_pre * 1e3 - 0.1, 10)


# plot two RMS over sound pressure level curves (plot (c) and (d))
for k in range(2):
    ax = fig_w.add_subplot(3, 2, k + 3)
    if k == 0:
        # load the data
        rms = np.loadtxt('RMS_amplitudes_2kHz.txt')
        params = np.loadtxt('RMS_amplitudes_2kHz_fit_parameters.txt')
        # set the y label
        ax.set_ylabel('RMS (mV)')
        ax.set_ylim(0, 10)
        ax.set_yticks([0, 1, 2, 3, 4, 5, 6, 7, 8, 9][::2])
    else:
        # load the data
        rms = np.loadtxt('RMS_amplitudes_4kHz.txt')
        params = np.loadtxt('RMS_amplitudes_4kHz_fit_parameters.txt')
        # set the y label
        ax.set_ylim(0, 10)
        ax.set_yticks([0, 1, 2, 3, 4, 5, 6, 7, 8, 9][::2])
        ax.set_yticklabels([])

    # plot the fit
    ax.plot(np.arange(-25, 180, 1),
            hard_sigmoid_with_noiselevel(np.arange(-25, 180, 1), params[0], params[1], params[2], params[3]) * 1000, color=color,
            linewidth=1, alpha=1)
    # plot the line for the background noise level
    ax.plot([-25, 180], np.array([params[3], params[3]]) * 1000, linestyle='--', color="k", lw=0.8)

    # plot the data points
    ax.plot(rms[:, 0], rms[:, 1] * 1000, marker='s', color=color, linewidth=0, ms=3)

    # set the x label
    ax.set_xlabel('sound pressure level (dB SPL)')
    ax.set_xlim(0, 150)

# add the plot for the threshold for different frequencies
ax_spec = fig_w.add_subplot(3, 2, 5)

# load the data
audiogram = np.loadtxt('audiogram_1-8kHz.txt')
# plot the boxes
bp2 = ax_spec.boxplot(np.transpose(audiogram[:, 3::]), positions=audiogram[:, 0], whis=[5, 95], widths=np.array((200, 400, 800, 1600)) * 0.5, showfliers=False)
# style the boxes
plt.setp(bp2['boxes'], color='c', linewidth=2)
plt.setp(bp2['caps'], color='c')
plt.setp(bp2['caps'], linewidth=2)
plt.setp(bp2['whiskers'], color='c', linewidth=2)
plt.setp(bp2['whiskers'], color='c')
plt.setp(bp2['fliers'], color='c')
plt.setp(bp2['medians'], color='r', linewidth=2)
# plot the medians
ax_spec.plot(audiogram[:, 0], np.median(audiogram[:, 3::], axis=1), color='r', label='subsampled data')
# plot the un subsampled data
ax_spec.plot(audiogram[:, 0], audiogram[:, 2], 'k--', label='no subsampling')

# seth the y limit
ax_spec.set_ylim(9, 90)
# convert the axis to a semilogarithmic plot
plt.semilogx([])
# set the x ticks
plt.xticks([1000, 2000, 4000, 8000], ["1", "2", "4", "8"])
plt.minorticks_off()

# set the x limits
ax_spec.set_xlim(900, 8900)
ax_spec.set_xlabel('frequency (kHz)')
ax_spec.set_ylabel('threshold\n(dB SPL)')
ax_spec.legend(loc=4)

# add letters to the subplots
add_letters()

#% start: automatic generated code from pylustrator
fig = plt.figure(1)
import matplotlib as mpl
fig.ax_dict = {ax.get_label(): ax for ax in fig.axes}
fig.set_size_inches(16.000000/2.54, 10.500000/2.54, forward=True)
fig.axes[4].get_legend()._set_loc((1.079594, 0.269009))
fig.axes[0].set_zorder(2)
fig.axes[0].text(0.5, 0.5, 'New Text', transform=fig.axes[0].transAxes)
fig.axes[0].set_position([0.131529, 0.698726, 0.374427, 0.253467])
fig.axes[0].spines['right'].set_visible(False)
fig.axes[0].spines['top'].set_visible(False)
fig.axes[0].texts[0].set_text("2 kHz")
fig.axes[0].texts[0].set_position([0.945069, 1.011028])  # Text: "2 kHz"
fig.axes[0].texts[0].set_ha("right")
fig.axes[1].text(0.5, 0.5, 'New Text', transform=fig.axes[1].transAxes)
fig.axes[1].text(0.5, 0.5, 'New Text', transform=fig.axes[1].transAxes, rotation=-90)
fig.axes[1].set_zorder(2)
fig.axes[1].set_position([0.557083, 0.696336, 0.393439, 0.253467])
fig.axes[1].spines['right'].set_visible(False)
fig.axes[1].spines['top'].set_visible(False)
fig.axes[1].texts[0].set_text("4 kHz")
fig.axes[1].texts[0].set_ha("right")
fig.axes[1].texts[0].set_position([0.943843, 1.010913])  # Text: "4 kHz"
fig.axes[1].texts[1].set_fontsize(8)
fig.axes[1].texts[1].set_ha("center")
fig.axes[1].texts[1].set_position([1.040682, 0.415574])  # Text: "50 mV"
fig.axes[1].texts[1].set_text("50 mV")
fig.axes[2].set_position([0.131529, 0.401056, 0.363288, 0.217368])
fig.axes[2].spines['right'].set_visible(False)
fig.axes[2].spines['top'].set_visible(False)
fig.axes[3].set_position([0.557083, 0.401056, 0.393439, 0.217415])
fig.axes[3].spines['right'].set_visible(False)
fig.axes[3].spines['top'].set_visible(False)
fig.axes[4].set_zorder(-1)
fig.axes[4].set_position([0.299224, 0.106663, 0.298226, 0.161515])
fig.axes[4].spines['right'].set_visible(False)
fig.axes[4].spines['top'].set_visible(False)
fig.axes[4].texts[0].set_position([0.085189, 0.954989])
fig.axes[4].texts[1].set_position([0.533862, 0.952569])
fig.axes[4].texts[2].set_position([0.085189, 0.610084])  # Text: "c"
fig.axes[4].texts[3].set_position([0.533862, 0.610084])
fig.axes[4].texts[4].set_position([0.178738, 0.252994])
#% end: automatic generated code from pylustrator

plt.savefig(__file__[:-3]+".pdf", dpi=300)
plt.savefig(__file__[:-3]+".png", dpi=300)

plt.show()
