# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
import pylustrator

font = 10
letter_fontsize = font

mpl.rc('xtick', labelsize=font)
mpl.rc('ytick', labelsize=font)
mpl.rc('legend', fontsize=font)
mpl.rc('axes', labelsize=font, titlesize=font)
mpl.rc('mathtext', default="regular")
mpl.rc('figure', dpi=100)
mpl.rc('legend', frameon=False)
mpl.rc('font', family="Helvetica Neue")

letter_index = 0
def add_letter(ax = None, offset=0, offset2=0, letter=None):
    global letter_index

    if ax is None:
        ax = plt.gca()

    box = ax.get_position()
    x_off = (-0.5/2.54)/plt.gcf().get_figwidth()+offset
    if letter is None:
        letter = chr(ord("a") + letter_index)
        letter_index += 1
    plt.text(box.x0+x_off, box.y1+offset2, letter, fontproperties=get_letter_font_prop(), va="bottom", ha="center", transform=plt.gcf().transFigure, picker=True)

def get_letter_font_prop():
    from matplotlib.font_manager import FontProperties
    font = FontProperties()
    font.set_family("Helvetica Neue")
    font.set_weight("bold")
    font.set_size(letter_fontsize)
    return font

def add_letters(*args, **kwargs):
    for ax in plt.gcf().axes:
        add_letter(ax, *args, **kwargs)

np.random.seed(1234)

def good_box(bp, color):
    for box in bp['boxes']:
        box.set(color=color, linewidth=1)
    for w in bp['whiskers']:
        w.set(color=color, linewidth=1)
    for f in bp['fliers']:
        f.set(mec="gray", alpha=0.5, linewidth=1, ms=1)
    for m in bp['medians']:
        m.set(linewidth=2, color="C3")
    for c in bp['caps']:
        c.set(linewidth=1, color='k')


def transform_att2loudness(data_load):
    data_load[:, 0] = 100 - (data_load[:, 0])  # for cortex 100dB reference
    return data_load


def hard_sigmoid(x, p1, p2, p3):
    return p1 * (x - p2) * (x >= p2) * (p1 * (x - p2) < p3) + p3 * (p1 * (x - p2) >= p3)


def hard_sigmoid_with_noiselevel(x, p1, p2, p3, const):
    return hard_sigmoid(x, p1, p2, p3) + const

# load the data
data_load = np.loadtxt('Auditory_Spikes_2kHz.txt')
data = transform_att2loudness(data_load)

# initialize the figure
fig_p = plt.figure(figsize=(16, 12))
ax_p = fig_p.add_subplot(221)

# get all stimulus levels
level = np.unique(data[:, 0])

cmap = pylustrator.LabColormap(("c", "b"), len(level))
color = cmap(len(level)//2)
# iterate over all levels
for i in range(len(level)):
    # find the data
    data_temp = data[data[:, 0] == level[i], 1::]
    # iterate over the data
    for j in range(len(data_temp)):
        # plot the points
        data_plot = data_temp[j, data_temp[j, :] > -1]
        ax_p.plot(data_plot * 1000, np.zeros(len(data_plot)) + 0.05 * j + 20 * i, 'bo', markersize=1, alpha=0.2, color=cmap(i))

# set labels and ticks
ax_p.set_ylim(-10, 260)
ax_p.set_ylabel('sound pressure level (dB SPL)')
ax_p.set_xlabel('time (ms)')
ax_p.plot([200, 200], [-10, 260], '--k', lw=0.8)
ax_p.set_yticks(np.arange(7.5, 20 * (i + 1) + 7.5, 20))
ax_p.set_yticklabels(level.astype("int"))

# Plot b, the spikerates over the sound level
means = np.loadtxt('spike_rate.txt')
params = np.loadtxt('spike_rate_parameters.txt')

ax = fig_p.add_subplot(222)

# plot the fitted line
ax.plot(np.arange(0, 100, 1), hard_sigmoid_with_noiselevel(np.arange(0, 100, 1), params[0], params[1], params[2], params[3]),
        color=color, linewidth=1, alpha=1)  # werte werden mal 1000 genommen

# plot the data points
ax.plot(means[:, 0], means[:, 1], marker='s', color=color, linewidth=0, ms=3)

# plot the base level
ax.plot([0, 90], np.array([params[3], params[3]]), linestyle='--', color="k", lw=0.8)

# plot the threshold
ax.plot([params[1], params[1]], np.array([0, 25]), linestyle='--', color="k", lw=0.8)

# labels and ticks
ax.set_ylabel('spikerate (1/s)')
ax.set_xlabel('sound level (dB SPL)')
ax.set_xlim(0, 100)
ax.set_ylim(0, 25)


# Plot c, plot the thresholds
ax0 = fig_p.add_subplot(224)

# load the data
trial_sample = np.arange(30, 290, 10)
trial_number_threshs = np.loadtxt('spike_thresholds.txt')

# add the boxes
bp = ax0.boxplot(np.transpose(trial_number_threshs), whis=[5, 95], widths=4, positions=trial_sample)
good_box(bp, color)

# ticks and labels
ax0.set_xlim(30, max(trial_sample) + 10)
ax0.set_xticks(trial_sample, minor=True)
ax0.set_xticks(trial_sample[2::5])
ax0.set_xticklabels(trial_sample[2::5])
ax0.set_xlabel('sample size')
ax0.set_ylabel('threshold (dB SPL)')
ax0.set_ylim(20, 60)

add_letters()

#% start: automatic generated code from pylustrator
fig = plt.figure(1)
import matplotlib as mpl
fig.ax_dict = {ax.get_label(): ax for ax in fig.axes}
fig.set_size_inches(16.000000/2.54, 12.000000/2.54, forward=True)
fig.axes[0].set_position([0.125000, 0.110000, 0.352273, 0.873814])
fig.axes[0].spines['right'].set_visible(False)
fig.axes[0].spines['top'].set_visible(False)
fig.axes[1].set_position([0.585883, 0.610509, 0.352273, 0.350000])
fig.axes[1].spines['right'].set_visible(False)
fig.axes[1].spines['top'].set_visible(False)
fig.axes[2].set_position([0.585883, 0.143315, 0.352273, 0.350000])
fig.axes[2].spines['right'].set_visible(False)
fig.axes[2].spines['top'].set_visible(False)
fig.axes[2].texts[0].set_position([0.071492, 0.960509])  # Text: "a"
fig.axes[2].texts[1].set_position([0.527606, 0.960509])  # Text: "b"
fig.axes[2].texts[2].set_position([0.527606, 0.491196])  # Text: "c"
#% end: automatic generated code from pylustrator

plt.savefig(__file__[:-3]+".pdf", dpi=300)
plt.savefig(__file__[:-3]+".png", dpi=300)

plt.show()
